/*
 *
 * This file contains routines related to  mep algorithm
 *
 * Diana Palsetia
 * 
 * Copyright (C) 2016, Northwestern University
 * See COPYRIGHT notice in top-level directory.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <malloc.h>
#include <unistd.h>
#include <sys/time.h>
#include "pmep.h"

int init_data_with_file(char   *filename,
                        idx_t   type,
                        idx_t  *direction,
                        idx_t  *num_edges,
                        idx_t **edges)
{
	int binary_fd=0;
	FILE *fd=NULL;
	char string[MAX_STRING_LENGTH];
	int ret=0;
	ssize_t read_ret;
	char file_fullname[MAX_STRING_LENGTH];

	if(type == 0){/* binary file */
		sprintf(file_fullname,"%s_edges.bin", filename);
		binary_fd = open(file_fullname, O_RDONLY);
		if(binary_fd == -1){
			printf("[%s][%d] open() failed\n", __FUNCTION__,__LINE__);
			return -1;
		}

		read_ret = read(binary_fd, num_edges, sizeof(idx_t));
		if(read_ret != sizeof(idx_t)){
			printf("[%s][%d] read failed\n", __FUNCTION__,__LINE__);
			ret = -1;
			goto out;
		}
		
		read_ret = read(binary_fd, &direction, sizeof(idx_t));
		if(read_ret != sizeof(idx_t)){
			printf("[%s][%d] read failed\n", __FUNCTION__,__LINE__);
			ret = -1;
			goto out;
		}
		
		if(*direction > 2)
			*direction = 1;

		*edges = (idx_t *)malloc(sizeof(idx_t)*(*num_edges)*2);
		if(*edges == NULL){
			printf("[%s][%d] out of memory\n", __FUNCTION__,__LINE__);
			ret = -1;
			goto out;
		}

		read_ret = read(binary_fd, *edges, sizeof(idx_t)*(*num_edges)*2);
		if(read_ret != sizeof(idx_t)*(*num_edges)*2){
			printf("[%s][%d] read failed\n", __FUNCTION__,__LINE__);
			ret = -1;
			goto out;
		}
	}
	else if(type == 1){/* txt file */
		char *temp=NULL;

		fd = fopen(filename, "r");
		if(fd == NULL){
			printf("[%s][%d] fopen() failed\n", __FUNCTION__,__LINE__);
			return -1;
		}

		fgets(string, MAX_STRING_LENGTH, fd);
		temp = strtok(string, ",\t ");
		if(temp == NULL){
			printf("[%s][%d] invalid input file format\n", __FUNCTION__,__LINE__);
			ret = -1;
			goto out;
		}
		(*num_edges) = atoi(temp);
	
		temp = strtok(NULL, ",\t ");
		if(temp == NULL){
			printf("[%s][%d] invalid input file format\n", __FUNCTION__,__LINE__);
			ret = -1;
			goto out;
		}
		*direction = atoi(temp);

		if(*direction > 2)
			*direction = 1;
		*edges = (idx_t *)malloc(sizeof(idx_t)*(*num_edges)*2);
		if(*edges == NULL){
			printf("[%s][%d] out of memory\n", __FUNCTION__,__LINE__);
			ret = -1;
			goto out;
		}

		int i=0;
		while(fgets(string, MAX_STRING_LENGTH, fd) != NULL){
			temp = strtok(string, ",\t ");
			if(temp == NULL){
				printf("[%s][%d] invalid format\n", __FUNCTION__,__LINE__);
				ret = -1;
				goto out;
			}	
			(*edges)[i++] = atoi(temp);

			temp = strtok(NULL, ",\t ");
			if(temp == NULL){
				printf("[%s][%d] invalid format\n", __FUNCTION__,__LINE__);
				ret = -1;
				goto out;
			}	
			(*edges)[i++] = atoi(temp);
		}
	}
	else{
		printf("[%s][%d] type is invalid, %d\n", __FUNCTION__,__LINE__, type);
		ret = -1;
		goto out;
	}

out:
	if(binary_fd != 0)
		close(binary_fd);

	if(fd != NULL)
		fclose(fd);

	return ret;
}

