/*
 *
 * This file contains routines related to parallel mep algorithm
 *
 * Diana Palsetia
 * 
 * Copyright (C) 2016, Northwestern University
 * See COPYRIGHT notice in top-level directory.
 *
 */
 
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include "pmep.h"

int countingSort(idx_t *a, idx_t *b, idx_t num_items, idx_t direction)
{
	int i=0;
	idx_t *count=NULL;
	idx_t max_value=0;
	idx_t total=0;
	idx_t oldCount=0;
	idx_t *data=NULL;
	idx_t *extra=NULL;

	if(a == NULL){
		printf("[%s][%d] invalid input...\n", __FUNCTION__,__LINE__);
		return -1;
	}

	data = (idx_t *)malloc(sizeof(idx_t)*num_items);
	if(data == NULL){
		printf("[%s][%d] out of memory\n", __FUNCTION__,__LINE__);
		return -1;
	}
	memcpy(data, a, sizeof(idx_t)*num_items);
	
	if(b != NULL){
		extra = (idx_t *)malloc(sizeof(idx_t)*num_items);
		if(extra == NULL){
			printf("[%s][%d] out of memory\n", __FUNCTION__,__LINE__);
			return -1;
		}
		memcpy(extra, b, sizeof(idx_t)*num_items);
	}

/* 1. find the maximum value */
	max_value = data[0];
	for(i=1; i<num_items; i++){
		if(max_value < data[i])
			max_value = data[i];
	}
	
	count = (idx_t *)malloc(sizeof(idx_t)*(max_value+1));
	if(count == NULL){
		printf("[%s][%d] out of memory\n", __FUNCTION__,__LINE__);
		return -1;
	}
	memset(count, 0, sizeof(idx_t)*(max_value+1));
	
/* 2. calculate the histogram of key frequencies */
	for(i=0 ;i<num_items; i++){
		count[data[i]] += 1;
	}

/* 3. calculate the staring index for each key */
	for(i=0; i<max_value+1; i++){
		oldCount = count[i];
		count[i] = total;
		total += oldCount;

		if(direction != 0){
			count[i] += oldCount;
			if(count[i] > 0)
				count[i] -= 1;
		}
	}

/* 4. copy to output array, preserving order of inputs with equal keys */
	for(i=0; i<num_items; i++){
		if(direction == 0){
			a[count[data[i]]] = data[i];
			if(extra != NULL){
				b[count[data[i]]] = extra[i];
			}
			count[data[i]] += 1;
		}
		else{
			a[num_items - count[data[i]] - 1] = data[i];
			if(extra != NULL){
				b[num_items - count[data[i]] - 1] = extra[i];
			}
			count[data[i]] -= 1;
		}
	}

	free(data);
	free(extra);
	return 0;
}
